#include "hobot_can_hal.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#define MAX_RX_BUF_SIZE 4000
#define BSWAP_32(x) \
	(uint32_t)((((uint32_t)(x) & 0xff000000) >> 24) | \
	(((uint32_t)(x) & 0x00ff0000) >> 8) | \
	(((uint32_t)(x) & 0x0000ff00) << 8) | \
	(((uint32_t)(x) & 0x000000ff) << 24) \
)
typedef struct {
  char *target;
  int canid;
} test_param;

static void help_info(const char *proc) {
  printf(
	  "Usage: %s target canid\n"
	  "    example target: bypass 2\n",
	  proc);
}

void *send_frame_data(void *arg)
{
	int ret = 0;
	test_param *test_params = (test_param *)arg;
	struct pack_info pack = {0};
	struct timeval tv;
	uint8_t frame_cnt[7] = {0,0,0,0,0,0,0};
	uint8_t frame_rc[7] = {0,0,0,0,0,0,0};
	uint8_t Can_au8Sdu8bytes[64U] = { 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa };
	uint32_t delay = 10; // ms
	struct canframe frame[1];
	uint32_t frame_num = 1;
	uint32_t canid = 0;    // 2: use can channel number
	uint32_t package_total = 0;
	uint32_t frame_total = 0;
	int all_frame_num = 0;
	void *rets = NULL;

	canid = test_params->canid;
	pack.data_num = frame_num;
	pack.length = pack.data_num;

	for (uint32_t i = 0; i < pack.data_num; i++) {
		gettimeofday(&tv, NULL);
		frame[i].time_stamp = (tv.tv_sec * 1000000 + tv.tv_usec) / 1000;
		frame[i].canid = BSWAP_32(0x131 + i);
		frame[i].count = frame_cnt[i];
		frame[i].can_channel = canid;
		if (i < 4) {
			frame[i].len = 8;
		} else {
			frame[i].len = 64;
		 }
		frame[i].can_type = (frame[i].len == 8) ? 0 : 1;    // 0: 8, 1: 64
		memcpy(frame[i].data, Can_au8Sdu8bytes, frame[i].len);
		//rolling counter
		if (frame_rc[i] > 15) {
			frame_rc[i] = 0;
		}
		frame[i].data[0] = frame_rc[i];
		//crc
		frame[i].data[7] = 0;
		for (int j = 0; j < 7; j++) {
			frame[i].data[7] += frame[i].data[j];
		}
		frame_total++;
		frame_cnt[i]++;
		frame_rc[i]++;

	}
	package_total++;

	ret = canSendMsgFrame(test_params->target, &frame[0], &pack);
	if (ret < 0) {
		printf("canSendMsgFrame failed ret: %d\n", ret);
	}
	printf("Send end, send package total: %d frame total: %d \n", package_total, frame_total);

	return rets;
}


int main(int argc, char *argv[])
{
	int ret = 0;
	char target[16] = {0};
	char action[16] = {0};
	test_param test_params;
	int canid = 0;
	if (argc < 3) {
		help_info(argv[0]);
		return 0;
	}
	strcpy(target, argv[1]);
	if (argv[2] != NULL) {
		canid = atoi(argv[2]);
	}
	test_params.target = target;
	test_params.canid = canid;
	ret = canInit();
	if (ret < 0)
	{
		printf("INIT ERROR!\n");
		return ret;
	}
	send_frame_data(&test_params);
	canDeInit();
	return 0;
}

